# Generated by Django 4.2.14 on 2024-07-25 02:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("register", "0016_attendee_affiliation"),
    ]

    operations = [
        migrations.CreateModel(
            name="TravelInsurance",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("surname", models.CharField(max_length=100)),
                ("given_names", models.CharField(max_length=100)),
                ("country", models.CharField(max_length=2)),
                ("passport_no", models.CharField(max_length=100)),
                ("gender", models.CharField(max_length=20)),
                ("date_of_birth", models.DateField()),
                (
                    "attendee",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="travel_insurance",
                        to="register.attendee",
                    ),
                ),
            ],
        ),
    ]
